/*********************************************************************
* Noddy program to adjust the outputs of a 12 channel DAC, part no.  *
* MB88346 situated on the controller board of a Samsung VP-L90       *
* LCD viewfinder. Expected connexions to the PC parallel port are: - *
*                                                                    *
* DB25 Pin  (Ref)   DAC Fn  Pin # (at LCD ribbon)                    *
*   2        D0      DI      2                                       *
*   3        D1      CLK     3                                       *
*   4        D2      LD      1                                       *
*  25        GND     GND     9, 10, 11, 17, 18                       *
*                                                                    *
* note: probably a good idea to protect three data lines with series *
*       resistors, eg. 680 ohms, just in case something goes wrong.  *
*                                                                    *
*                                                        05/05/2008  *
*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <linux/ioctl.h>
#include <linux/parport.h>
#include <linux/ppdev.h>
#include <fcntl.h>

char Usage[] =
	"level-tweak - utility for sending commands to the MB88346 IC\n"
	"found on board a Samsung camcorder LCD\n\n"
	"usage: level-tweak <channel> <level>\n"
	"  <channel> is a value between 1 and 12\n"
	"  <level> is a value between 0 and 255\n\n"
	"channels:\n"
	"   1 - R-Sub\n"
	"   2 - B-Sub\n"
	"   3 - Tint\n"
	"   4 - Colour\n"
	"   5 - Brightness\n"
	"   6 - Mirror horizontal\n"
	"   7 - Mirror vertical \n"
	"   8 - \"CONT\"\n"
	"   9 - Pin 5 voltage\n"
	"  10 - Gamma\n"
	"  11 - Widescreen mode\n"
	"  12 - Unused\n\n";

int main (int argc, char *argv[])
{
	int Channel;
	int Level;
	int fd, mode, i;
	unsigned char data;

	if (argc != 3)
	{
		printf (Usage);
		exit (1);
	}
	
	sscanf (argv[1], "%ud", &Channel);
	sscanf (argv[2], "%ud", &Level);
	
	if (Channel < 1 || Channel > 12)
	{
		printf ("<channel> must be between 1 and 12\n");
		exit (1);
	}
	Level &= 0xFF;
	
	printf ("Setting channel %d to %d.\n", Channel, Level);
	
	fd = open ("/dev/parport0",O_RDWR);
	if (fd == -1)
	{
		perror ("open");
		exit (1);
	}


	if (ioctl(fd,PPCLAIM))
	{
		perror ("PPCLAIM");
		close (fd);
		exit (1);
	}


	mode = IEEE1284_MODE_COMPAT;
	if (ioctl(fd, PPNEGOT, &mode))
	{
		perror ("PPNEGOT");
		close (fd);
		exit (1);
	}

	data = 0xF8;

	for (i = 0x08; i != 0; i >>= 1)
	{
		data &= ~0x01;
		data |= (Channel & i)? 0x01: 0x00;
		ioctl (fd, PPWDATA, &data);

		data |= 0x02;
		ioctl (fd, PPWDATA, &data);
		
		data &= ~0x02;
		ioctl (fd, PPWDATA, &data);
	}
	
	for (i = 0x80; i != 0; i >>= 1)
	{
		data &= ~0x01;
		data |= (Level & i)? 0x01: 0x00;
		ioctl (fd, PPWDATA, &data);

		data |= 0x02;
		ioctl (fd, PPWDATA, &data);
		
		data &= ~0x02;
		ioctl (fd, PPWDATA, &data);
	}
	
	data |= 0x04;
	ioctl (fd, PPWDATA, &data);
	
	data &= ~0x04;
	ioctl (fd, PPWDATA, &data);
	
	ioctl (fd, PPRELEASE);

	close(fd);
	
	return 0;
}

